import math
from readability.text.analyzer import Analyzer
from readability.exceptions import ReadabilityException


class Result:
    def __init__(self, score, grade_level):
        self.score = score
        self.grade_level = grade_level

    def __str__(self):
        return "score: {}, grade_level: {}". \
            format(self.score, self.grade_level)


class Smog:
    def __init__(self, stats, sentences):
        if stats.num_sentences < 30:
            raise ReadabilityException(
                'SMOG requires 30 sentences. {} found'
                .format(stats.num_sentences))

        self._stats = stats
        self._smog_stats = self._smog_text_stats(sentences)

    def score(self):
        score = self._score()
        grade_level = self._grade_level(score)
        return Result(
            score=score,
            grade_level=grade_level
        )

    def _score(self):
        smog_stats = self._smog_stats
        num_complex_words = smog_stats.num_poly_syllable_words
        num_sentences = 30
        return 1.0430 * math.sqrt(30 * num_complex_words / num_sentences) + 3.1291

    def _grade_level(self, score):
        return str(round(score))

    def _smog_text_stats(self, sentences):
        mid = int(math.floor(len(sentences) / 2))
        first_10_sents = sentences[:10]
        mid_10_sents = sentences[mid-5: mid+5]
        last_10_sents = sentences[-10:]

        smog_text = ' '.join(first_10_sents + mid_10_sents + last_10_sents)
        return Analyzer().analyze(smog_text)



import math
from readability.exceptions import ReadabilityException


class Result:
    def __init__(self, score, grade_levels, ages):
        self.score = score
        self.grade_levels = grade_levels
        self.ages = ages

    def __str__(self):
        return "score: {}, grade_levels: {}, ages: {}". \
            format(self.score, self.grade_levels, self.ages)


class ARI:
    def __init__(self, stats):
        self._stats = stats
        if stats.num_words < 100:
            raise ReadabilityException('100 words required.')

    def score(self):
        score = self._score()
        return Result(
            score=score,
            grade_levels=self._grade_levels(score),
            ages=self._ages(score))

    def _score(self):
        s = self._stats
        letters_per_word = s.num_letters / s.num_words
        words_per_sent = s.num_words / s.num_sentences
        return 4.71 * letters_per_word + 0.5 * words_per_sent - 21.43

    def _grade_levels(self, score):
        score = math.ceil(score)
        if score <= 1:
            return ['K']
        elif score <= 2:
            return ['1', '2']
        elif score <= 3:
            return ['3']
        elif score <= 4:
            return ['4']
        elif score <= 5:
            return ['5']
        elif score <= 6:
            return ['6']
        elif score <= 7:
            return ['7']
        elif score <= 8:
            return ['8']
        elif score <= 9:
            return ['9']
        elif score <= 10:
            return ['10']
        elif score <= 11:
            return ['11']
        elif score <= 12:
            return ['12']
        elif score <= 13:
            return ['college']
        else:
            return ['college_graduate']

    def _ages(self, score):
        score = math.ceil(score)
        if score <= 1:
            return [5, 6]
        elif score <= 2:
            return [6, 7]
        elif score <= 3:
            return [7, 9]
        elif score <= 4:
            return [9, 10]
        elif score <= 5:
            return [10, 11]
        elif score <= 6:
            return [11, 12]
        elif score <= 7:
            return [12, 13]
        elif score <= 8:
            return [13, 14]
        elif score <= 9:
            return [14, 15]
        elif score <= 10:
            return [15, 16]
        elif score <= 11:
            return [16, 17]
        elif score <= 12:
            return [17, 18]
        elif score <= 13:
            return [18, 24]
        else:
            return [24, 100]



import re


def count(word):
    """
    Simple syllable counting
    """

    word = word if type(word) is str else str(word)

    word = word.lower()

    if len(word) <= 3:
        return 1

    word = re.sub('(?:[^laeiouy]es|[^laeiouy]e)$', '', word) # removed ed|
    word = re.sub('^y', '', word)
    matches = re.findall('[aeiouy]{1,2}', word)
    return len(matches)



from readability.exceptions import ReadabilityException


class Result:
    def __init__(self, score, grade_levels, ease):
        self.score = score
        self.ease = ease
        self.grade_levels = grade_levels

    def __str__(self):
        return "score: {}, ease: '{}', grade_levels: {}". \
            format(self.score, self.ease, self.grade_levels)


class Flesch:
    def __init__(self, stats):
        self._stats = stats
        if stats.num_words < 100:
            raise ReadabilityException('100 words required.')

    def score(self):
        score = self._score()
        return Result(
            score=score,
            ease=self._ease(score),
            grade_levels=self._grade_levels(score))

    def _score(self):
        stats = self._stats
        words_per_sent = stats.num_words / stats.num_sentences
        syllables_per_word = stats.num_syllables / stats.num_words
        return 206.835 - (1.015 * words_per_sent) - (84.6 * syllables_per_word)

    def _ease(self, score):
        if score >= 90 and score <= 100:
            return 'very_easy'
        elif score >= 80 and score < 90:
            return 'easy'
        elif score >= 70 and score < 80:
            return 'fairly_easy'
        elif score >= 60 and score < 70:
            return 'standard'
        elif score >= 50 and score < 60:
            return 'fairly_difficult'
        elif score >= 30 and score < 50:
            return 'difficult'
        else:
            return 'very_confusing'

    def _grade_levels(self, score):
        if score >= 90 and score <= 100:
            return ['5']
        elif score >= 80 and score < 90:
            return ['6']
        elif score >= 70 and score < 80:
            return ['7']
        elif score >= 60 and score < 70:
            return ['8', '9']
        elif score >= 50 and score < 60:
            return ['10', '11', '12']
        elif score >= 30 and score < 50:
            return ['college']
        else:
            return ['college_graduate']



from .text import Analyzer
from .scorers import ARI, ColemanLiau, DaleChall, Flesch, \
    FleschKincaid, GunningFog, LinsearWrite, Smog


class Readability:
    def __init__(self, text):
        self._analyzer = Analyzer()
        self._statistics = self._analyzer.analyze(text)

    def ari(self):
        """Calculate Automated Readability Index (ARI)."""
        return ARI(self._statistics).score()

    def coleman_liau(self):
        """Calculate Coleman Liau Index."""
        return ColemanLiau(self._statistics).score()

    def dale_chall(self):
        """Calculate Dale Chall."""
        return DaleChall(self._statistics).score()

    def flesch(self):
        """Calculate Flesch Reading Ease score."""
        return Flesch(self._statistics).score()

    def flesch_kincaid(self):
        """Calculate Flesch-Kincaid Grade Level."""
        return FleschKincaid(self._statistics).score()

    def gunning_fog(self):
        """Calculate Gunning Fog score."""
        return GunningFog(self._statistics).score()

    def linsear_write(self):
        """Calculate Linsear Write."""
        return LinsearWrite(self._statistics).score()

    def smog(self):
        """SMOG Index."""
        return Smog(self._statistics, self._analyzer.sentences).score()

    def statistics(self):
        return {
            'num_letters': self._statistics.num_letters,
            'num_words': self._statistics.num_words,
            'num_sentences': self._statistics.num_sentences,
            'num_polysyllabic_words': self._statistics.num_poly_syllable_words,
            'avg_words_per_sentence': self._statistics.avg_words_per_sentence,
            'avg_syllables_per_word': self._statistics.avg_syllables_per_word,
        }
